"""
.. _ref_unit_system:

UnitSystem
===============
"""
from ansys.dpf import core as dpf
from ansys.dpf.core import errors as dpf_errors
from ansys.dpf.core import server as server_module


class UnitSystem:
    """Defines an internally coherent way of measuring units.

    Notes
    -----
    Class available with server's version starting at 6.1 (Ansys 2023R2).
    """

    def __init__(self, name, ID=None, unit_names=None):
        """
        Creates a new UnitSystem from its name and its Ansys ID

        Parameters
        ----------
        name: string
            name of the Unit System (compulsory)
        ID: int
            internal Ansys ID that identifies the Unit System (optional)
        unit_names: string
            semicolon-separated string with the base units (Length, Mass, Time, Temperature,
            Electric Charge and Angle) (optional).

        ID and unit_names are mutually exclusionary, but one of them should
        be provided (either one or the other, but not both)

        Examples
        --------
        Create an Ansys default UnitSystem (solver_mks)

        >>> from ansys.dpf import core as dpf
        >>> my_unit_system = dpf.UnitSystem("solver_mks", ID=11)

        Create a customized version of the mks Unit System that measures
        Temperature in Fahrenheit

        >>> from ansys.dpf import core as dpf
        >>> my_unit_system = dpf.UnitSystem("my_mks", unit_names="m;kg;s;degF;C;rad")
        """
        server = server_module.get_or_create_server(None)
        if server and not server.meet_version("6.1"):  # pragma: no cover
            raise dpf_errors.DpfVersionNotSupported("6.1")
        if not isinstance(name, str):
            raise dpf_errors.InvalidTypeError("str", "name")

        if ID is not None and unit_names is None:
            if not isinstance(ID, int):
                raise dpf_errors.InvalidTypeError("int", "ID")
            self._name = name
            self._ID = ID
            self._unit_names = ""
        elif ID is None and unit_names is not None:
            if not isinstance(unit_names, str):
                raise dpf_errors.InvalidTypeError("str", "unit_names")
            unit_system_check = dpf.Operator("check_unit_system")
            unit_system_check.connect(0, unit_names)
            unit_system_isok = unit_system_check.get_output(0, bool)
            if unit_system_isok:
                self._name = name
                self._ID = -2
                self._unit_names = unit_names
            else:
                raise Exception(unit_system_check.outputs.get_output(1, str))

        else:
            raise Exception(
                "ID and unit_names are mutually exclusionary, but one of them should be provided."
            )

    @property
    def ID(self) -> int:
        return self._ID

    @property
    def name(self) -> str:
        return self._name

    @property
    def unit_names(self) -> str:
        if self._unit_names == "":  # Ansys UnitSystem
            unit_system_strings = dpf.Operator("unit_system_strings")
            unit_system_strings.connect(0, self._ID)
            return unit_system_strings.get_output(0, str)
        else:  # Custom UnitSystem
            return self._unit_names


class unit_systems:
    """Contains common Ansys predefined UnitSystems.

    Notes
    -----
    Class available with server's version starting at 6.1 (Ansys 2023R2).

    Attributes
    -----------
    solver_mks : Metric (m, kg, N, s, J, Pa, degC, C, rad)

    solver_cgs : Metric (cm, g, dyne, s, erg, dyne*cm^-2, degC, C, rad)

    solver_nmm : Metric (mm, ton, N, s, mJ, MPa, degC, mC, rad)

    solver_umks : Metric (um, kg, uN, s, pJ, MPa, degC, pC, rad)

    solver_knms : Metric (mm, kg, kN, ms, J, GPa, degC, mC, rad)

    solver_bft : U.S. Customary (ft, slug, lbf, s, ft*lbf, lbf*ft^-2, degF, C, rad)

    solver_bin : U.S. Customary (in, slinch, lbf, s, in*lbf, lbf*in^-2, degF, C, rad)

    undefined : All units are dimensionless

    """

    try:
        solver_mks = UnitSystem("solver_mks", ID=11)
        solver_cgs = UnitSystem("solver_cgs", ID=5)
        solver_nmm = UnitSystem("solver_nmm", ID=6)
        solver_umks = UnitSystem("solver_umks", ID=10)
        solver_knms = UnitSystem("solver_knms", ID=16)
        solver_bft = UnitSystem("solver_bft", ID=7)
        solver_bin = UnitSystem("solver_bin", ID=8)
        undefined = UnitSystem("undefined", ID=-1)
    except dpf_errors.DpfVersionNotSupported as e:  # pragma: no cover
        pass



"""Streamlines computation specific helpers.
"""

import numpy as np
import warnings

from ansys.dpf.core.common import locations
from ansys.dpf.core.fields_container import FieldsContainer
from ansys.dpf.core.helpers.utils import _sort_supported_kwargs


class _PvFieldsContainerBase:
    def __init__(self, data):
        """Instantiate Streamline
        from pyvista.PolyData object.
        This construction is only
        intended to be used internally.

        Parameters
        ----------
        pv_data_set: pyvista.PolyData
        """
        try:
            import pyvista as pv
        except ModuleNotFoundError:
            raise ModuleNotFoundError(
                "To use streamlines capabilities, please install pyvista "
                "with :\n pip install pyvista>=0.24.0"
            )
        self._pv_data_set = None
        self._streamlines_fc = None
        if isinstance(data, pv.PolyData):
            self._pv_data_set = data
        elif isinstance(FieldsContainer):
            self._streamlines_fc = data
        else:
            raise AttributeError(
                "streamlines must be a pyvista.PolyData or a dpf.FieldsContainer instance."
            )

    def _pv_data_set_to_fc(self):
        """Convert pyvista.PolyData into FieldsContainer."""
        raise Exception("Not implemented yet")

    def _fc_to_pv_data_set(self):
        """Convert FieldsContainer into pyvista.PolyData."""
        raise Exception("Not implemented yet")

    def _as_pyvista_data_set(self):
        if self._pv_data_set is None:
            self._pv_data_set = self._fc_to_pv_data_set()
        return self._pv_data_set

    def _as_fields_container(self):
        """Returns a FieldsContainer representing the streamlines
        related objects.

        Returns
        -------
        streamlines_fields_container : FieldsContainer
            FieldsContainer containing the streamlines data array,
            and a MeshedRegion as support.

        """
        # once implemented, this method will become public
        if self._streamlines_fc is None:
            self._streamlines_fc = self._pv_data_set_to_fc()
        return self._streamlines_fc


class Streamlines(_PvFieldsContainerBase):
    """Class to define the Streamlines object
    scripting with `ansys-dpf-core`.

    """

    def __init__(self, data):
        super().__init__(data=data)


class StreamlinesSource(_PvFieldsContainerBase):
    """Class to define the StreamlinesSource
    object scripting with `ansys-dpf-core`.

    """

    def __init__(self, data):
        super().__init__(data=data)


def compute_streamlines(meshed_region, field, **kwargs):
    """Compute the streamlines for a given mesh and velocity
    field.

    Parameters
    ----------
    meshed_region: MeshedRegion
        MeshedRegion the streamline will be computed on.
    field: Field
        Field containing raw vector data the streamline is
        computed from. The data location must be nodal, velocity
        values must be defined at nodes.
    **kwargs : optional
        Additional keyword arguments for the streamline
        computation. More information is available at
        :func:`pyvista.DataSetFilters.streamlines`.

    Returns
    -------
    streamlines: helpers.streamlines.Streamlines

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> from ansys.dpf.core.helpers.streamlines import compute_streamlines
    >>> # Get model and meshed region
    >>> files = examples.download_fluent_mixing_elbow_steady_state()
    >>> ds = dpf.DataSources()
    >>> ds.set_result_file_path(files["cas"][0], "cas")
    >>> ds.add_file_path(files["dat"][1], "dat")
    >>> model = dpf.Model(ds)
    >>> mesh = model.metadata.meshed_region
    >>> # Get velocity data
    >>> velocity_op = model.results.velocity()
    >>> fc = velocity_op.outputs.fields_container()
    >>> op = dpf.operators.averaging.to_nodal_fc(fields_container=fc)
    >>> field = op.outputs.fields_container()[0]
    >>> # compute streamline
    >>> streamline_obj = compute_streamlines(
    ...        meshed_region=mesh,
    ...        field=field,
    ...        source_center=(0.55, 0.55, 0.),
    ...        n_points=10,
    ...        source_radius=0.08,
    ...        max_time=10.0
    ...        )

    """
    # Check velocity field location
    if field.location is not locations.nodal:
        warnings.warn(
            "Velocity field must have a nodal location. Result must be carefully checked."
        )

    # handles input data
    f_name = field.name
    stream_name = "streamlines " + f_name + " (" + str(field.unit) + ")"
    grid = meshed_region.grid
    mesh_nodes = meshed_region.nodes

    ind, mask = mesh_nodes.map_scoping(field.scoping)
    overall_data = np.full((len(mesh_nodes), 3), np.nan)  # velocity has 3 components
    overall_data[ind] = field.data[mask]

    grid.set_active_scalars(None)
    grid[f"{stream_name}"] = overall_data

    # check src request
    return_source = kwargs.pop("return_source", None)

    # filter kwargs
    kwargs_base = _sort_supported_kwargs(bound_method=grid.streamlines, **kwargs)
    kwargs_from_source = _sort_supported_kwargs(bound_method=grid.streamlines_from_source, **kwargs)
    kwargs_from_source.update(kwargs_base)  # merge both dicts in kwargs_from_source

    if return_source:
        streamlines, src = grid.streamlines(
            vectors=f"{stream_name}",
            return_source=True,
            **kwargs_from_source,
        )
        streamlines = Streamlines(data=streamlines)
        src = StreamlinesSource(data=src)
        return streamlines, src
    else:
        streamlines = grid.streamlines(
            vectors=f"{stream_name}",
            **kwargs_from_source,
        )
        streamlines = Streamlines(data=streamlines)
        return streamlines



"""
cyclic_expansion
================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class cyclic_expansion(Operator):
    """Expand cyclic results from a fieldsContainer for given sets, sectors
    and scoping (optionals).

    Parameters
    ----------
    time_scoping : Scoping, optional
    mesh_scoping : ScopingsContainer or Scoping, optional
    fields_container : FieldsContainer
        Field container with the base and duplicate
        sectors
    bool_rotate_to_global : bool, optional
        Default is true
    map_size_scoping_out : optional
        Map provider by scoping adapter
    merge_stages : bool, optional
    cyclic_support : CyclicSupport
    sectors_to_expand : Scoping or ScopingsContainer, optional
        Sectors to expand (start at 0), for
        multistage: use scopings container
        with 'stage' label.
    phi : float, optional
        Angle phi in degrees (default value 0.0)


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.cyclic_expansion()

    >>> # Make input connections
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.ScopingsContainer()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_bool_rotate_to_global = bool()
    >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
    >>> my_map_size_scoping_out = dpf.()
    >>> op.inputs.map_size_scoping_out.connect(my_map_size_scoping_out)
    >>> my_merge_stages = bool()
    >>> op.inputs.merge_stages.connect(my_merge_stages)
    >>> my_cyclic_support = dpf.CyclicSupport()
    >>> op.inputs.cyclic_support.connect(my_cyclic_support)
    >>> my_sectors_to_expand = dpf.Scoping()
    >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
    >>> my_phi = float()
    >>> op.inputs.phi.connect(my_phi)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.cyclic_expansion(
    ...     time_scoping=my_time_scoping,
    ...     mesh_scoping=my_mesh_scoping,
    ...     fields_container=my_fields_container,
    ...     bool_rotate_to_global=my_bool_rotate_to_global,
    ...     map_size_scoping_out=my_map_size_scoping_out,
    ...     merge_stages=my_merge_stages,
    ...     cyclic_support=my_cyclic_support,
    ...     sectors_to_expand=my_sectors_to_expand,
    ...     phi=my_phi,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        time_scoping=None,
        mesh_scoping=None,
        fields_container=None,
        bool_rotate_to_global=None,
        map_size_scoping_out=None,
        merge_stages=None,
        cyclic_support=None,
        sectors_to_expand=None,
        phi=None,
        config=None,
        server=None,
    ):
        super().__init__(name="cyclic_expansion", config=config, server=server)
        self._inputs = InputsCyclicExpansion(self)
        self._outputs = OutputsCyclicExpansion(self)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if bool_rotate_to_global is not None:
            self.inputs.bool_rotate_to_global.connect(bool_rotate_to_global)
        if map_size_scoping_out is not None:
            self.inputs.map_size_scoping_out.connect(map_size_scoping_out)
        if merge_stages is not None:
            self.inputs.merge_stages.connect(merge_stages)
        if cyclic_support is not None:
            self.inputs.cyclic_support.connect(cyclic_support)
        if sectors_to_expand is not None:
            self.inputs.sectors_to_expand.connect(sectors_to_expand)
        if phi is not None:
            self.inputs.phi.connect(phi)

    @staticmethod
    def _spec():
        description = """Expand cyclic results from a fieldsContainer for given sets, sectors
            and scoping (optionals)."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_scoping",
                    type_names=["scoping", "vector<int32>"],
                    optional=True,
                    document="""""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scopings_container", "scoping", "vector<int32>"],
                    optional=True,
                    document="""""",
                ),
                2: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Field container with the base and duplicate
        sectors""",
                ),
                5: PinSpecification(
                    name="bool_rotate_to_global",
                    type_names=["bool"],
                    optional=True,
                    document="""Default is true""",
                ),
                6: PinSpecification(
                    name="map_size_scoping_out",
                    type_names=["umap<int32,int32>"],
                    optional=True,
                    document="""Map provider by scoping adapter""",
                ),
                14: PinSpecification(
                    name="merge_stages",
                    type_names=["bool"],
                    optional=True,
                    document="""""",
                ),
                16: PinSpecification(
                    name="cyclic_support",
                    type_names=["cyclic_support"],
                    optional=False,
                    document="""""",
                ),
                18: PinSpecification(
                    name="sectors_to_expand",
                    type_names=["vector<int32>", "scoping", "scopings_container"],
                    optional=True,
                    document="""Sectors to expand (start at 0), for
        multistage: use scopings container
        with 'stage' label.""",
                ),
                19: PinSpecification(
                    name="phi",
                    type_names=["double"],
                    optional=True,
                    document="""Angle phi in degrees (default value 0.0)""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainer filled in""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="cyclic_expansion", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsCyclicExpansion
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsCyclicExpansion
        """
        return super().outputs


class InputsCyclicExpansion(_Inputs):
    """Intermediate class used to connect user inputs to
    cyclic_expansion operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.cyclic_expansion()
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.ScopingsContainer()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_bool_rotate_to_global = bool()
    >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
    >>> my_map_size_scoping_out = dpf.()
    >>> op.inputs.map_size_scoping_out.connect(my_map_size_scoping_out)
    >>> my_merge_stages = bool()
    >>> op.inputs.merge_stages.connect(my_merge_stages)
    >>> my_cyclic_support = dpf.CyclicSupport()
    >>> op.inputs.cyclic_support.connect(my_cyclic_support)
    >>> my_sectors_to_expand = dpf.Scoping()
    >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
    >>> my_phi = float()
    >>> op.inputs.phi.connect(my_phi)
    """

    def __init__(self, op: Operator):
        super().__init__(cyclic_expansion._spec().inputs, op)
        self._time_scoping = Input(cyclic_expansion._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(cyclic_expansion._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh_scoping)
        self._fields_container = Input(cyclic_expansion._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._fields_container)
        self._bool_rotate_to_global = Input(
            cyclic_expansion._spec().input_pin(5), 5, op, -1
        )
        self._inputs.append(self._bool_rotate_to_global)
        self._map_size_scoping_out = Input(
            cyclic_expansion._spec().input_pin(6), 6, op, -1
        )
        self._inputs.append(self._map_size_scoping_out)
        self._merge_stages = Input(cyclic_expansion._spec().input_pin(14), 14, op, -1)
        self._inputs.append(self._merge_stages)
        self._cyclic_support = Input(cyclic_expansion._spec().input_pin(16), 16, op, -1)
        self._inputs.append(self._cyclic_support)
        self._sectors_to_expand = Input(
            cyclic_expansion._spec().input_pin(18), 18, op, -1
        )
        self._inputs.append(self._sectors_to_expand)
        self._phi = Input(cyclic_expansion._spec().input_pin(19), 19, op, -1)
        self._inputs.append(self._phi)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Parameters
        ----------
        my_time_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Parameters
        ----------
        my_mesh_scoping : ScopingsContainer or Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator.

        Field container with the base and duplicate
        sectors

        Parameters
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def bool_rotate_to_global(self):
        """Allows to connect bool_rotate_to_global input to the operator.

        Default is true

        Parameters
        ----------
        my_bool_rotate_to_global : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
        >>> # or
        >>> op.inputs.bool_rotate_to_global(my_bool_rotate_to_global)
        """
        return self._bool_rotate_to_global

    @property
    def map_size_scoping_out(self):
        """Allows to connect map_size_scoping_out input to the operator.

        Map provider by scoping adapter

        Parameters
        ----------
        my_map_size_scoping_out :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.map_size_scoping_out.connect(my_map_size_scoping_out)
        >>> # or
        >>> op.inputs.map_size_scoping_out(my_map_size_scoping_out)
        """
        return self._map_size_scoping_out

    @property
    def merge_stages(self):
        """Allows to connect merge_stages input to the operator.

        Parameters
        ----------
        my_merge_stages : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.merge_stages.connect(my_merge_stages)
        >>> # or
        >>> op.inputs.merge_stages(my_merge_stages)
        """
        return self._merge_stages

    @property
    def cyclic_support(self):
        """Allows to connect cyclic_support input to the operator.

        Parameters
        ----------
        my_cyclic_support : CyclicSupport

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.cyclic_support.connect(my_cyclic_support)
        >>> # or
        >>> op.inputs.cyclic_support(my_cyclic_support)
        """
        return self._cyclic_support

    @property
    def sectors_to_expand(self):
        """Allows to connect sectors_to_expand input to the operator.

        Sectors to expand (start at 0), for
        multistage: use scopings container
        with 'stage' label.

        Parameters
        ----------
        my_sectors_to_expand : Scoping or ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
        >>> # or
        >>> op.inputs.sectors_to_expand(my_sectors_to_expand)
        """
        return self._sectors_to_expand

    @property
    def phi(self):
        """Allows to connect phi input to the operator.

        Angle phi in degrees (default value 0.0)

        Parameters
        ----------
        my_phi : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> op.inputs.phi.connect(my_phi)
        >>> # or
        >>> op.inputs.phi(my_phi)
        """
        return self._phi


class OutputsCyclicExpansion(_Outputs):
    """Intermediate class used to get outputs from
    cyclic_expansion operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.cyclic_expansion()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(cyclic_expansion._spec().outputs, op)
        self._fields_container = Output(cyclic_expansion._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cyclic_expansion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
merge_result_infos
==================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class merge_result_infos(Operator):
    """Assembles a set of result information into a unique one.

    Parameters
    ----------
    result_infos1 : ResultInfo
        A vector of result info containers to merge
        or result infos from pin 0 to ...
    result_infos2 : ResultInfo
        A vector of result info containers to merge
        or result infos from pin 0 to ...


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.merge_result_infos()

    >>> # Make input connections
    >>> my_result_infos1 = dpf.ResultInfo()
    >>> op.inputs.result_infos1.connect(my_result_infos1)
    >>> my_result_infos2 = dpf.ResultInfo()
    >>> op.inputs.result_infos2.connect(my_result_infos2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.merge_result_infos(
    ...     result_infos1=my_result_infos1,
    ...     result_infos2=my_result_infos2,
    ... )

    >>> # Get output data
    >>> result_merged_result_infos = op.outputs.merged_result_infos()
    """

    def __init__(
        self, result_infos1=None, result_infos2=None, config=None, server=None
    ):
        super().__init__(name="merge::result_info", config=config, server=server)
        self._inputs = InputsMergeResultInfos(self)
        self._outputs = OutputsMergeResultInfos(self)
        if result_infos1 is not None:
            self.inputs.result_infos1.connect(result_infos1)
        if result_infos2 is not None:
            self.inputs.result_infos2.connect(result_infos2)

    @staticmethod
    def _spec():
        description = """Assembles a set of result information into a unique one."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="result_infos",
                    type_names=["result_info"],
                    optional=False,
                    document="""A vector of result info containers to merge
        or result infos from pin 0 to ...""",
                ),
                1: PinSpecification(
                    name="result_infos",
                    type_names=["result_info"],
                    optional=False,
                    document="""A vector of result info containers to merge
        or result infos from pin 0 to ...""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="merged_result_infos",
                    type_names=["result_info"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="merge::result_info", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMergeResultInfos
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMergeResultInfos
        """
        return super().outputs


class InputsMergeResultInfos(_Inputs):
    """Intermediate class used to connect user inputs to
    merge_result_infos operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_result_infos()
    >>> my_result_infos1 = dpf.ResultInfo()
    >>> op.inputs.result_infos1.connect(my_result_infos1)
    >>> my_result_infos2 = dpf.ResultInfo()
    >>> op.inputs.result_infos2.connect(my_result_infos2)
    """

    def __init__(self, op: Operator):
        super().__init__(merge_result_infos._spec().inputs, op)
        self._result_infos1 = Input(merge_result_infos._spec().input_pin(0), 0, op, 0)
        self._inputs.append(self._result_infos1)
        self._result_infos2 = Input(merge_result_infos._spec().input_pin(1), 1, op, 1)
        self._inputs.append(self._result_infos2)

    @property
    def result_infos1(self):
        """Allows to connect result_infos1 input to the operator.

        A vector of result info containers to merge
        or result infos from pin 0 to ...

        Parameters
        ----------
        my_result_infos1 : ResultInfo

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_result_infos()
        >>> op.inputs.result_infos1.connect(my_result_infos1)
        >>> # or
        >>> op.inputs.result_infos1(my_result_infos1)
        """
        return self._result_infos1

    @property
    def result_infos2(self):
        """Allows to connect result_infos2 input to the operator.

        A vector of result info containers to merge
        or result infos from pin 0 to ...

        Parameters
        ----------
        my_result_infos2 : ResultInfo

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_result_infos()
        >>> op.inputs.result_infos2.connect(my_result_infos2)
        >>> # or
        >>> op.inputs.result_infos2(my_result_infos2)
        """
        return self._result_infos2


class OutputsMergeResultInfos(_Outputs):
    """Intermediate class used to get outputs from
    merge_result_infos operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_result_infos()
    >>> # Connect inputs : op.inputs. ...
    >>> result_merged_result_infos = op.outputs.merged_result_infos()
    """

    def __init__(self, op: Operator):
        super().__init__(merge_result_infos._spec().outputs, op)
        self._merged_result_infos = Output(
            merge_result_infos._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._merged_result_infos)

    @property
    def merged_result_infos(self):
        """Allows to get merged_result_infos output of the operator

        Returns
        ----------
        my_merged_result_infos : ResultInfo

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_result_infos()
        >>> # Connect inputs : op.inputs. ...
        >>> result_merged_result_infos = op.outputs.merged_result_infos()
        """  # noqa: E501
        return self._merged_result_infos



"""
migrate_to_h5dpf
================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class migrate_to_h5dpf(Operator):
    """Read mesh properties from the results files contained in the streams
    or data sources and make those properties available through a mesh
    selection manager in output.

    Parameters
    ----------
    filename : str
        Filename of the migrated file
    comma_separated_list_of_results : str, optional
        List of result (source operator names) that
        will be stored. if empty, all
        available results will be converted.
    all_time_sets : bool, optional
        Default is true
    streams_container : StreamsContainer, optional
        Streams (result file container) (optional)
    data_sources : DataSources, optional
        If the stream is null then we need to get the
        file path from the data sources


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.migrate_to_h5dpf()

    >>> # Make input connections
    >>> my_filename = str()
    >>> op.inputs.filename.connect(my_filename)
    >>> my_comma_separated_list_of_results = str()
    >>> op.inputs.comma_separated_list_of_results.connect(my_comma_separated_list_of_results)
    >>> my_all_time_sets = bool()
    >>> op.inputs.all_time_sets.connect(my_all_time_sets)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.migrate_to_h5dpf(
    ...     filename=my_filename,
    ...     comma_separated_list_of_results=my_comma_separated_list_of_results,
    ...     all_time_sets=my_all_time_sets,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_migrated_file = op.outputs.migrated_file()
    """

    def __init__(
        self,
        filename=None,
        comma_separated_list_of_results=None,
        all_time_sets=None,
        streams_container=None,
        data_sources=None,
        config=None,
        server=None,
    ):
        super().__init__(name="hdf5::h5dpf::migrate_file", config=config, server=server)
        self._inputs = InputsMigrateToH5Dpf(self)
        self._outputs = OutputsMigrateToH5Dpf(self)
        if filename is not None:
            self.inputs.filename.connect(filename)
        if comma_separated_list_of_results is not None:
            self.inputs.comma_separated_list_of_results.connect(
                comma_separated_list_of_results
            )
        if all_time_sets is not None:
            self.inputs.all_time_sets.connect(all_time_sets)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        description = """Read mesh properties from the results files contained in the streams
            or data sources and make those properties available
            through a mesh selection manager in output."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="filename",
                    type_names=["string"],
                    optional=False,
                    document="""Filename of the migrated file""",
                ),
                1: PinSpecification(
                    name="comma_separated_list_of_results",
                    type_names=["string"],
                    optional=True,
                    document="""List of result (source operator names) that
        will be stored. if empty, all
        available results will be converted.""",
                ),
                2: PinSpecification(
                    name="all_time_sets",
                    type_names=["bool"],
                    optional=True,
                    document="""Default is true""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Streams (result file container) (optional)""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=True,
                    document="""If the stream is null then we need to get the
        file path from the data sources""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="migrated_file",
                    type_names=["data_sources"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="hdf5::h5dpf::migrate_file", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMigrateToH5Dpf
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMigrateToH5Dpf
        """
        return super().outputs


class InputsMigrateToH5Dpf(_Inputs):
    """Intermediate class used to connect user inputs to
    migrate_to_h5dpf operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.migrate_to_h5dpf()
    >>> my_filename = str()
    >>> op.inputs.filename.connect(my_filename)
    >>> my_comma_separated_list_of_results = str()
    >>> op.inputs.comma_separated_list_of_results.connect(my_comma_separated_list_of_results)
    >>> my_all_time_sets = bool()
    >>> op.inputs.all_time_sets.connect(my_all_time_sets)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(migrate_to_h5dpf._spec().inputs, op)
        self._filename = Input(migrate_to_h5dpf._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._filename)
        self._comma_separated_list_of_results = Input(
            migrate_to_h5dpf._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._comma_separated_list_of_results)
        self._all_time_sets = Input(migrate_to_h5dpf._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._all_time_sets)
        self._streams_container = Input(
            migrate_to_h5dpf._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._streams_container)
        self._data_sources = Input(migrate_to_h5dpf._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def filename(self):
        """Allows to connect filename input to the operator.

        Filename of the migrated file

        Parameters
        ----------
        my_filename : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.filename.connect(my_filename)
        >>> # or
        >>> op.inputs.filename(my_filename)
        """
        return self._filename

    @property
    def comma_separated_list_of_results(self):
        """Allows to connect comma_separated_list_of_results input to the operator.

        List of result (source operator names) that
        will be stored. if empty, all
        available results will be converted.

        Parameters
        ----------
        my_comma_separated_list_of_results : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.comma_separated_list_of_results.connect(my_comma_separated_list_of_results)
        >>> # or
        >>> op.inputs.comma_separated_list_of_results(my_comma_separated_list_of_results)
        """
        return self._comma_separated_list_of_results

    @property
    def all_time_sets(self):
        """Allows to connect all_time_sets input to the operator.

        Default is true

        Parameters
        ----------
        my_all_time_sets : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.all_time_sets.connect(my_all_time_sets)
        >>> # or
        >>> op.inputs.all_time_sets(my_all_time_sets)
        """
        return self._all_time_sets

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator.

        Streams (result file container) (optional)

        Parameters
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        If the stream is null then we need to get the
        file path from the data sources

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsMigrateToH5Dpf(_Outputs):
    """Intermediate class used to get outputs from
    migrate_to_h5dpf operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.migrate_to_h5dpf()
    >>> # Connect inputs : op.inputs. ...
    >>> result_migrated_file = op.outputs.migrated_file()
    """

    def __init__(self, op: Operator):
        super().__init__(migrate_to_h5dpf._spec().outputs, op)
        self._migrated_file = Output(migrate_to_h5dpf._spec().output_pin(0), 0, op)
        self._outputs.append(self._migrated_file)

    @property
    def migrated_file(self):
        """Allows to get migrated_file output of the operator

        Returns
        ----------
        my_migrated_file : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.migrate_to_h5dpf()
        >>> # Connect inputs : op.inputs. ...
        >>> result_migrated_file = op.outputs.migrated_file()
        """  # noqa: E501
        return self._migrated_file



"""
streams_provider
================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class streams_provider(Operator):
    """Creates streams (files with cache) from the data sources.

    Parameters
    ----------
    data_sources : DataSources


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.metadata.streams_provider()

    >>> # Make input connections
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.metadata.streams_provider(
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_streams_container = op.outputs.streams_container()
    """

    def __init__(self, data_sources=None, config=None, server=None):
        super().__init__(name="stream_provider", config=config, server=server)
        self._inputs = InputsStreamsProvider(self)
        self._outputs = OutputsStreamsProvider(self)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        description = """Creates streams (files with cache) from the data sources."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="stream_provider", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsStreamsProvider
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsStreamsProvider
        """
        return super().outputs


class InputsStreamsProvider(_Inputs):
    """Intermediate class used to connect user inputs to
    streams_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.metadata.streams_provider()
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(streams_provider._spec().inputs, op)
        self._data_sources = Input(streams_provider._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.streams_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsStreamsProvider(_Outputs):
    """Intermediate class used to get outputs from
    streams_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.metadata.streams_provider()
    >>> # Connect inputs : op.inputs. ...
    >>> result_streams_container = op.outputs.streams_container()
    """

    def __init__(self, op: Operator):
        super().__init__(streams_provider._spec().outputs, op)
        self._streams_container = Output(streams_provider._spec().output_pin(0), 0, op)
        self._outputs.append(self._streams_container)

    @property
    def streams_container(self):
        """Allows to get streams_container output of the operator

        Returns
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.streams_provider()
        >>> # Connect inputs : op.inputs. ...
        >>> result_streams_container = op.outputs.streams_container()
        """  # noqa: E501
        return self._streams_container



"""
txt_file_to_dpf
===============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class txt_file_to_dpf(Operator):
    """Take an input string and parse it into dataProcessing type.

    Parameters
    ----------
    input_string : str
        Ex: 'double:1.0', 'int:1',
        'vector<double>:1.0;1.0'


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.txt_file_to_dpf()

    >>> # Make input connections
    >>> my_input_string = str()
    >>> op.inputs.input_string.connect(my_input_string)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.txt_file_to_dpf(
    ...     input_string=my_input_string,
    ... )

    >>> # Get output data
    >>> result_any_output1 = op.outputs.any_output1()
    >>> result_any_output2 = op.outputs.any_output2()
    """

    def __init__(self, input_string=None, config=None, server=None):
        super().__init__(name="text_parser", config=config, server=server)
        self._inputs = InputsTxtFileToDpf(self)
        self._outputs = OutputsTxtFileToDpf(self)
        if input_string is not None:
            self.inputs.input_string.connect(input_string)

    @staticmethod
    def _spec():
        description = """Take an input string and parse it into dataProcessing type."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="input_string",
                    type_names=["string"],
                    optional=False,
                    document="""Ex: 'double:1.0', 'int:1',
        'vector<double>:1.0;1.0'""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="any_output1",
                    optional=False,
                    document="""Any output""",
                ),
                1: PinSpecification(
                    name="any_output2",
                    optional=False,
                    document="""Any output""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="text_parser", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsTxtFileToDpf
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsTxtFileToDpf
        """
        return super().outputs


class InputsTxtFileToDpf(_Inputs):
    """Intermediate class used to connect user inputs to
    txt_file_to_dpf operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.txt_file_to_dpf()
    >>> my_input_string = str()
    >>> op.inputs.input_string.connect(my_input_string)
    """

    def __init__(self, op: Operator):
        super().__init__(txt_file_to_dpf._spec().inputs, op)
        self._input_string = Input(txt_file_to_dpf._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._input_string)

    @property
    def input_string(self):
        """Allows to connect input_string input to the operator.

        Ex: 'double:1.0', 'int:1',
        'vector<double>:1.0;1.0'

        Parameters
        ----------
        my_input_string : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.txt_file_to_dpf()
        >>> op.inputs.input_string.connect(my_input_string)
        >>> # or
        >>> op.inputs.input_string(my_input_string)
        """
        return self._input_string


class OutputsTxtFileToDpf(_Outputs):
    """Intermediate class used to get outputs from
    txt_file_to_dpf operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.txt_file_to_dpf()
    >>> # Connect inputs : op.inputs. ...
    >>> result_any_output1 = op.outputs.any_output1()
    >>> result_any_output2 = op.outputs.any_output2()
    """

    def __init__(self, op: Operator):
        super().__init__(txt_file_to_dpf._spec().outputs, op)
        self._any_output1 = Output(txt_file_to_dpf._spec().output_pin(0), 0, op)
        self._outputs.append(self._any_output1)
        self._any_output2 = Output(txt_file_to_dpf._spec().output_pin(1), 1, op)
        self._outputs.append(self._any_output2)

    @property
    def any_output1(self):
        """Allows to get any_output1 output of the operator

        Returns
        ----------
        my_any_output1 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.txt_file_to_dpf()
        >>> # Connect inputs : op.inputs. ...
        >>> result_any_output1 = op.outputs.any_output1()
        """  # noqa: E501
        return self._any_output1

    @property
    def any_output2(self):
        """Allows to get any_output2 output of the operator

        Returns
        ----------
        my_any_output2 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.txt_file_to_dpf()
        >>> # Connect inputs : op.inputs. ...
        >>> result_any_output2 = op.outputs.any_output2()
        """  # noqa: E501
        return self._any_output2



"""
.. _ref_custom_operator:

Custom Operator Base
====================
Contains utilities allowing you to implement and record custom Python operators.
"""

import abc
import ctypes
import numpy
import traceback

from ansys.dpf import core as dpf
from ansys.dpf.core import (
    settings,
    server,
    server_factory,
    operator_specification,
    dpf_operator,
    collection,
    AvailableServerContexts,
)
from ansys.dpf.core._custom_operators_helpers import (
    __operator_main__,
    functions_registry,
    external_operator_api,
    _type_to_output_method,
    _type_to_input_method,
)
from ansys.dpf.gate import object_handler, capi, dpf_vector, integral_types


def record_operator(operator_type, *args) -> None:
    """
    Add an operator (with its name, run callback, and specification) to the DPF core registry.

    Parameters
    ----------
    operator_type : type, CustomOperatorBase
        Class type inheriting from CustomOperatorBase.
        ``name`` and ``specification`` properties are called
        and run method callback is given to DataProcessingCore.

    *args
        Forwarded arguments passed in ``load_operators`` method

    """
    if isinstance(operator_type, type):
        operator = operator_type()
    else:
        operator = operator_type
    if dpf.SERVER is None:
        settings.set_server_configuration(server_factory.ServerConfig(None, False))
        server.start_local_server(context=AvailableServerContexts.premium)
    if len(args) == 2:
        external_operator_api.external_operator_record_with_abstract_core_and_wrapper(
            operator._call_back(),
            __operator_main__,
            operator.name,
            operator._internal_specification,
            ctypes.c_void_p(*args[0]),
            ctypes.c_void_p(*args[1]),
        )
    else:
        external_operator_api.external_operator_record_with_abstract_core(
            operator._call_back(),
            __operator_main__,
            operator.name,
            operator._internal_specification,
            ctypes.c_void_p(*args),
        )


class CustomOperatorBase:
    """
    Base class interfacing CPython Custom Operators which can be used as regular
    DPF Operators in any API.
    A CustomOperator is defined by its name, its specification and its run method.
    These three abstract methods should be implemented to create a CustomOperator.

    Examples
    --------
    Create a Custom Operator which adds an input float value to the data of an input Field.

    >>> from ansys.dpf.core.custom_operator import CustomOperatorBase
    >>> from ansys.dpf.core.operator_specification import CustomSpecification, \
    SpecificationProperties, PinSpecification
    >>> from ansys.dpf.core import Field
    >>> class AddFloatToFieldData(CustomOperatorBase):
    ...     def run(self):
    ...         field = self.get_input(0, Field)
    ...         to_add = self.get_input(1, float)
    ...         data = field.data
    ...         data += to_add
    ...         self.set_output(0, field)
    ...         self.set_succeeded()
    ...
    ...     @property
    ...     def specification(self):
    ...         spec = CustomSpecification()
    ...         spec.description = "Add a custom value to all the data of an input Field"
    ...         spec.inputs = {
    ...             0: PinSpecification("field", [Field], "Field on which float value is added."),
    ...             1: PinSpecification("to_add", [float], "Data to add.") }
    ...         spec.outputs = {
    ...             0: PinSpecification("field", [Field], "Updated field.")}
    ...         spec.properties = SpecificationProperties("custom add to field", "math")
    ...         return spec
    ...
    ...     @property
    ...     def name(self):
    ...         return "custom_add_to_field"

    And record it:

    >>> from ansys.dpf.core.custom_operator import record_operator
    >>> def load_operators(*args):
    ...     record_operator(AddFloatToFieldData, *args)

    """

    def set_output(self, index: int, data) -> None:
        """
        Add an output to this Operator at the given index.
        To use in the ``run`` method.

        Parameters
        ----------
        index : int
            Index of the output.

        data: int, float, Field, Scoping, DataSources, FieldsContainer...
            Data of any supported type to return.

        """
        for type_tuple in _type_to_output_method:
            if isinstance(data, type_tuple[0]):
                return type_tuple[1](self._operator_data, index, data)
        if isinstance(data, (list, numpy.ndarray)):
            data = collection.Collection.integral_collection(data, dpf.SERVER)
            return external_operator_api.external_operator_put_out_collection_as_vector(
                self._operator_data, index, data
            )
        raise TypeError(f"unable to set output of type {type(data).__name__}")

    def get_input(self, index, type: type):
        """
        Method used to get an input of a requested type at a given index in the ``run`` method.
        The correct input type must be connected to this Operator beforehand.

        Parameters
        ----------
        index : int
            Index of the input.

        type : type, :class:`ansys.dpf.core.common.types`
            Expected type of the data.

        Returns
        -------
        data: type
        """
        type = dpf_operator._write_output_type_to_type(type)
        for type_tuple in _type_to_input_method:
            if type is type_tuple[0]:
                if len(type_tuple) >= 3:
                    parameters = {type_tuple[2]: type_tuple[1](self._operator_data, index)}
                    return type(**parameters)
                return type(type_tuple[1](self._operator_data, index))
        if type == dpf_vector.DPFVectorInt:
            size = integral_types.MutableInt32(0)
            out = external_operator_api.external_operator_get_in_vec_int(
                self._operator_data, index, size
            )
            return numpy.ctypeslib.as_array(out, shape=(int(size),))
        raise TypeError(f"{type} is not a supported operator input")

    def set_failed(self) -> None:
        """
        Set the Operator's status to "failed".
        To use in the ``run`` method if an error occurred.
        This "failed" status is automatically set when an exception is raised in the ``run`` method.
        """
        external_operator_api.external_operator_put_status(self._operator_data, 1)

    def set_succeeded(self) -> None:
        """
        Set the Operator's status to "succeeded".
        To use at the end of the ``run`` method.
        """
        external_operator_api.external_operator_put_status(self._operator_data, 0)

    def _call_back(self):
        def _self_call_back(op):
            self._operator_data = op
            try:
                self.run()
            except:
                external_operator_api.external_operator_put_exception(
                    self._operator_data, 4, str(traceback.format_exc())
                )

        functions_registry.append(capi.OperatorMainCallback(_self_call_back))
        return functions_registry[-1]

    @property
    def _internal_specification(self):
        if isinstance(self.specification, operator_specification.Specification):
            return self.specification
        else:
            return object_handler.ObjHandler(ctypes.c_void_p(self.specification))

    @abc.abstractmethod
    def run(self) -> None:
        """
        Callback of the Operator to implement.
        The implementation should first request the inputs with the method ``get_input``,
        compute the output data, then add the outputs with the method ``set_output`` and finally
        call ``set_succeeded``.
        """
        pass

    @property
    @abc.abstractmethod
    def specification(self):
        """
        Documents the operator. The following are mandatory  to have a full support
        (documentation, code generation and usage) of the new operator:
        * Description
        * Supported inputs (a name, a document, a list of accepted types (optional) and/or ellipses)
        * Supported outputs (a name, a document, a type, and can be ellipsis)
        * User name
        * Category

        Returns
        -------
        spec : CustomSpecification
        """
        pass

    @property
    @abc.abstractmethod
    def name(self) -> str:
        """
        Returns the identifier or name of the operator.
        This name can then be used to instantiate the Operator.
        """
        pass



from .fft import fft 
from .find_reduced_coordinates import find_reduced_coordinates 
from .on_coordinates import on_coordinates 
from .on_reduced_coordinates import on_reduced_coordinates 
from .prep_sampling_fft import prep_sampling_fft 
from .prepare_mapping_workflow import prepare_mapping_workflow 
from .scoping_on_coordinates import scoping_on_coordinates 
from .solid_to_skin import solid_to_skin 
from .solid_to_skin_fc import solid_to_skin_fc 



"""
strain_eqv_as_mechanical
========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class strain_eqv_as_mechanical(Operator):
    """Computes the equivalent (Von Mises) elastic strains and averages it to
    the nodes (by default). For multibody simulations, averaging
    across bodies can either be activated or deactivated.

    Parameters
    ----------
    time_scoping : Scoping, optional
        Time/freq (use doubles or field), time/freq
        set ids (use ints or scoping) or
        time/freq step ids use scoping with
        timefreq_steps location) required in
        output.
    mesh_scoping : Scoping or ScopingsContainer, optional
        Nodes or elements scoping required in output.
    streams_container : StreamsContainer, optional
        Result file container allowed to be kept open
        to cache data.
    data_sources : DataSources
        Result file path container.
    mesh : MeshedRegion or MeshesContainer, optional
        Prevents from reading the mesh in the results
        file.
    requested_location : str, optional
        Average the elemental nodal result to the
        requested location (default is
        nodal).
    poisson_ratio : int or float, optional
        Poisson ratio to be used in equivalent strain
        calculation.
    read_cyclic : bool, optional
        If true, cyclic expansion is done. if false,
        it's ignored.
    average_across_bodies : bool, optional
        For multibody simulations, the stresses are
        averaged across bodies if true or not
        if false (default).


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.strain_eqv_as_mechanical()

    >>> # Make input connections
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_poisson_ratio = int()
    >>> op.inputs.poisson_ratio.connect(my_poisson_ratio)
    >>> my_read_cyclic = bool()
    >>> op.inputs.read_cyclic.connect(my_read_cyclic)
    >>> my_average_across_bodies = bool()
    >>> op.inputs.average_across_bodies.connect(my_average_across_bodies)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.strain_eqv_as_mechanical(
    ...     time_scoping=my_time_scoping,
    ...     mesh_scoping=my_mesh_scoping,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ...     mesh=my_mesh,
    ...     requested_location=my_requested_location,
    ...     poisson_ratio=my_poisson_ratio,
    ...     read_cyclic=my_read_cyclic,
    ...     average_across_bodies=my_average_across_bodies,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    >>> result_meshes_container = op.outputs.meshes_container()
    """

    def __init__(
        self,
        time_scoping=None,
        mesh_scoping=None,
        streams_container=None,
        data_sources=None,
        mesh=None,
        requested_location=None,
        poisson_ratio=None,
        read_cyclic=None,
        average_across_bodies=None,
        config=None,
        server=None,
    ):
        super().__init__(name="strain_eqv_as_mechanical", config=config, server=server)
        self._inputs = InputsStrainEqvAsMechanical(self)
        self._outputs = OutputsStrainEqvAsMechanical(self)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if requested_location is not None:
            self.inputs.requested_location.connect(requested_location)
        if poisson_ratio is not None:
            self.inputs.poisson_ratio.connect(poisson_ratio)
        if read_cyclic is not None:
            self.inputs.read_cyclic.connect(read_cyclic)
        if average_across_bodies is not None:
            self.inputs.average_across_bodies.connect(average_across_bodies)

    @staticmethod
    def _spec():
        description = """Computes the equivalent (Von Mises) elastic strains and averages it to
            the nodes (by default). For multibody simulations,
            averaging across bodies can either be activated or
            deactivated."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_scoping",
                    type_names=["scoping", "vector<int32>"],
                    optional=True,
                    document="""Time/freq (use doubles or field), time/freq
        set ids (use ints or scoping) or
        time/freq step ids use scoping with
        timefreq_steps location) required in
        output.""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping", "scopings_container"],
                    optional=True,
                    document="""Nodes or elements scoping required in output.""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Result file container allowed to be kept open
        to cache data.""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Result file path container.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region", "meshes_container"],
                    optional=True,
                    document="""Prevents from reading the mesh in the results
        file.""",
                ),
                9: PinSpecification(
                    name="requested_location",
                    type_names=["string"],
                    optional=True,
                    document="""Average the elemental nodal result to the
        requested location (default is
        nodal).""",
                ),
                13: PinSpecification(
                    name="poisson_ratio",
                    type_names=["int32", "double"],
                    optional=True,
                    document="""Poisson ratio to be used in equivalent strain
        calculation.""",
                ),
                14: PinSpecification(
                    name="read_cyclic",
                    type_names=["bool"],
                    optional=True,
                    document="""If true, cyclic expansion is done. if false,
        it's ignored.""",
                ),
                200: PinSpecification(
                    name="average_across_bodies",
                    type_names=["bool"],
                    optional=True,
                    document="""For multibody simulations, the stresses are
        averaged across bodies if true or not
        if false (default).""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="meshes_container",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="strain_eqv_as_mechanical", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsStrainEqvAsMechanical
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsStrainEqvAsMechanical
        """
        return super().outputs


class InputsStrainEqvAsMechanical(_Inputs):
    """Intermediate class used to connect user inputs to
    strain_eqv_as_mechanical operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.strain_eqv_as_mechanical()
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_poisson_ratio = int()
    >>> op.inputs.poisson_ratio.connect(my_poisson_ratio)
    >>> my_read_cyclic = bool()
    >>> op.inputs.read_cyclic.connect(my_read_cyclic)
    >>> my_average_across_bodies = bool()
    >>> op.inputs.average_across_bodies.connect(my_average_across_bodies)
    """

    def __init__(self, op: Operator):
        super().__init__(strain_eqv_as_mechanical._spec().inputs, op)
        self._time_scoping = Input(
            strain_eqv_as_mechanical._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(
            strain_eqv_as_mechanical._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mesh_scoping)
        self._streams_container = Input(
            strain_eqv_as_mechanical._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._streams_container)
        self._data_sources = Input(
            strain_eqv_as_mechanical._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)
        self._mesh = Input(strain_eqv_as_mechanical._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)
        self._requested_location = Input(
            strain_eqv_as_mechanical._spec().input_pin(9), 9, op, -1
        )
        self._inputs.append(self._requested_location)
        self._poisson_ratio = Input(
            strain_eqv_as_mechanical._spec().input_pin(13), 13, op, -1
        )
        self._inputs.append(self._poisson_ratio)
        self._read_cyclic = Input(
            strain_eqv_as_mechanical._spec().input_pin(14), 14, op, -1
        )
        self._inputs.append(self._read_cyclic)
        self._average_across_bodies = Input(
            strain_eqv_as_mechanical._spec().input_pin(200), 200, op, -1
        )
        self._inputs.append(self._average_across_bodies)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Time/freq (use doubles or field), time/freq
        set ids (use ints or scoping) or
        time/freq step ids use scoping with
        timefreq_steps location) required in
        output.

        Parameters
        ----------
        my_time_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Nodes or elements scoping required in output.

        Parameters
        ----------
        my_mesh_scoping : Scoping or ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator.

        Result file container allowed to be kept open
        to cache data.

        Parameters
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Result file path container.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Prevents from reading the mesh in the results
        file.

        Parameters
        ----------
        my_mesh : MeshedRegion or MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator.

        Average the elemental nodal result to the
        requested location (default is
        nodal).

        Parameters
        ----------
        my_requested_location : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> # or
        >>> op.inputs.requested_location(my_requested_location)
        """
        return self._requested_location

    @property
    def poisson_ratio(self):
        """Allows to connect poisson_ratio input to the operator.

        Poisson ratio to be used in equivalent strain
        calculation.

        Parameters
        ----------
        my_poisson_ratio : int or float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.poisson_ratio.connect(my_poisson_ratio)
        >>> # or
        >>> op.inputs.poisson_ratio(my_poisson_ratio)
        """
        return self._poisson_ratio

    @property
    def read_cyclic(self):
        """Allows to connect read_cyclic input to the operator.

        If true, cyclic expansion is done. if false,
        it's ignored.

        Parameters
        ----------
        my_read_cyclic : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.read_cyclic.connect(my_read_cyclic)
        >>> # or
        >>> op.inputs.read_cyclic(my_read_cyclic)
        """
        return self._read_cyclic

    @property
    def average_across_bodies(self):
        """Allows to connect average_across_bodies input to the operator.

        For multibody simulations, the stresses are
        averaged across bodies if true or not
        if false (default).

        Parameters
        ----------
        my_average_across_bodies : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> op.inputs.average_across_bodies.connect(my_average_across_bodies)
        >>> # or
        >>> op.inputs.average_across_bodies(my_average_across_bodies)
        """
        return self._average_across_bodies


class OutputsStrainEqvAsMechanical(_Outputs):
    """Intermediate class used to get outputs from
    strain_eqv_as_mechanical operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.strain_eqv_as_mechanical()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    >>> result_meshes_container = op.outputs.meshes_container()
    """

    def __init__(self, op: Operator):
        super().__init__(strain_eqv_as_mechanical._spec().outputs, op)
        self._fields_container = Output(
            strain_eqv_as_mechanical._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)
        self._meshes_container = Output(
            strain_eqv_as_mechanical._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._meshes_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container

    @property
    def meshes_container(self):
        """Allows to get meshes_container output of the operator

        Returns
        ----------
        my_meshes_container : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.strain_eqv_as_mechanical()
        >>> # Connect inputs : op.inputs. ...
        >>> result_meshes_container = op.outputs.meshes_container()
        """  # noqa: E501
        return self._meshes_container



"""
hdf5dpf_generate_result_file
============================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class hdf5dpf_generate_result_file(Operator):
    """Generate a dpf result file from provided information.

    Parameters
    ----------
    filename : str
        Name of the output file that will be
        generated (utf8).
    mesh_provider_out : MeshedRegion, optional
        Defines the meshedregion that is exported and
        provided by meshprovider.
    time_freq_support_out : TimeFreqSupport, optional
        Defines the timefreqsupport that is exported
        and provided by
        timefreqsupportprovider.
    ansys_unit_system_id : int, optional
        Defines the unitsystem the results are
        exported with.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()

    >>> # Make input connections
    >>> my_filename = str()
    >>> op.inputs.filename.connect(my_filename)
    >>> my_mesh_provider_out = dpf.MeshedRegion()
    >>> op.inputs.mesh_provider_out.connect(my_mesh_provider_out)
    >>> my_time_freq_support_out = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support_out.connect(my_time_freq_support_out)
    >>> my_ansys_unit_system_id = int()
    >>> op.inputs.ansys_unit_system_id.connect(my_ansys_unit_system_id)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file(
    ...     filename=my_filename,
    ...     mesh_provider_out=my_mesh_provider_out,
    ...     time_freq_support_out=my_time_freq_support_out,
    ...     ansys_unit_system_id=my_ansys_unit_system_id,
    ... )

    >>> # Get output data
    >>> result_data_sources = op.outputs.data_sources()
    """

    def __init__(
        self,
        filename=None,
        mesh_provider_out=None,
        time_freq_support_out=None,
        ansys_unit_system_id=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="hdf5::h5dpf::make_result_file", config=config, server=server
        )
        self._inputs = InputsHdf5DpfGenerateResultFile(self)
        self._outputs = OutputsHdf5DpfGenerateResultFile(self)
        if filename is not None:
            self.inputs.filename.connect(filename)
        if mesh_provider_out is not None:
            self.inputs.mesh_provider_out.connect(mesh_provider_out)
        if time_freq_support_out is not None:
            self.inputs.time_freq_support_out.connect(time_freq_support_out)
        if ansys_unit_system_id is not None:
            self.inputs.ansys_unit_system_id.connect(ansys_unit_system_id)

    @staticmethod
    def _spec():
        description = """Generate a dpf result file from provided information."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="filename",
                    type_names=["string"],
                    optional=False,
                    document="""Name of the output file that will be
        generated (utf8).""",
                ),
                1: PinSpecification(
                    name="mesh_provider_out",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""Defines the meshedregion that is exported and
        provided by meshprovider.""",
                ),
                2: PinSpecification(
                    name="time_freq_support_out",
                    type_names=["time_freq_support"],
                    optional=True,
                    document="""Defines the timefreqsupport that is exported
        and provided by
        timefreqsupportprovider.""",
                ),
                3: PinSpecification(
                    name="ansys_unit_system_id",
                    type_names=["int32"],
                    optional=True,
                    document="""Defines the unitsystem the results are
        exported with.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Data_sources filed with the h5 generated file
        path.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="hdf5::h5dpf::make_result_file", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsHdf5DpfGenerateResultFile
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsHdf5DpfGenerateResultFile
        """
        return super().outputs


class InputsHdf5DpfGenerateResultFile(_Inputs):
    """Intermediate class used to connect user inputs to
    hdf5dpf_generate_result_file operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()
    >>> my_filename = str()
    >>> op.inputs.filename.connect(my_filename)
    >>> my_mesh_provider_out = dpf.MeshedRegion()
    >>> op.inputs.mesh_provider_out.connect(my_mesh_provider_out)
    >>> my_time_freq_support_out = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support_out.connect(my_time_freq_support_out)
    >>> my_ansys_unit_system_id = int()
    >>> op.inputs.ansys_unit_system_id.connect(my_ansys_unit_system_id)
    """

    def __init__(self, op: Operator):
        super().__init__(hdf5dpf_generate_result_file._spec().inputs, op)
        self._filename = Input(
            hdf5dpf_generate_result_file._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._filename)
        self._mesh_provider_out = Input(
            hdf5dpf_generate_result_file._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mesh_provider_out)
        self._time_freq_support_out = Input(
            hdf5dpf_generate_result_file._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._time_freq_support_out)
        self._ansys_unit_system_id = Input(
            hdf5dpf_generate_result_file._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._ansys_unit_system_id)

    @property
    def filename(self):
        """Allows to connect filename input to the operator.

        Name of the output file that will be
        generated (utf8).

        Parameters
        ----------
        my_filename : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()
        >>> op.inputs.filename.connect(my_filename)
        >>> # or
        >>> op.inputs.filename(my_filename)
        """
        return self._filename

    @property
    def mesh_provider_out(self):
        """Allows to connect mesh_provider_out input to the operator.

        Defines the meshedregion that is exported and
        provided by meshprovider.

        Parameters
        ----------
        my_mesh_provider_out : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()
        >>> op.inputs.mesh_provider_out.connect(my_mesh_provider_out)
        >>> # or
        >>> op.inputs.mesh_provider_out(my_mesh_provider_out)
        """
        return self._mesh_provider_out

    @property
    def time_freq_support_out(self):
        """Allows to connect time_freq_support_out input to the operator.

        Defines the timefreqsupport that is exported
        and provided by
        timefreqsupportprovider.

        Parameters
        ----------
        my_time_freq_support_out : TimeFreqSupport

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()
        >>> op.inputs.time_freq_support_out.connect(my_time_freq_support_out)
        >>> # or
        >>> op.inputs.time_freq_support_out(my_time_freq_support_out)
        """
        return self._time_freq_support_out

    @property
    def ansys_unit_system_id(self):
        """Allows to connect ansys_unit_system_id input to the operator.

        Defines the unitsystem the results are
        exported with.

        Parameters
        ----------
        my_ansys_unit_system_id : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()
        >>> op.inputs.ansys_unit_system_id.connect(my_ansys_unit_system_id)
        >>> # or
        >>> op.inputs.ansys_unit_system_id(my_ansys_unit_system_id)
        """
        return self._ansys_unit_system_id


class OutputsHdf5DpfGenerateResultFile(_Outputs):
    """Intermediate class used to get outputs from
    hdf5dpf_generate_result_file operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()
    >>> # Connect inputs : op.inputs. ...
    >>> result_data_sources = op.outputs.data_sources()
    """

    def __init__(self, op: Operator):
        super().__init__(hdf5dpf_generate_result_file._spec().outputs, op)
        self._data_sources = Output(
            hdf5dpf_generate_result_file._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._data_sources)

    @property
    def data_sources(self):
        """Allows to get data_sources output of the operator

        Returns
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_generate_result_file()
        >>> # Connect inputs : op.inputs. ...
        >>> result_data_sources = op.outputs.data_sources()
        """  # noqa: E501
        return self._data_sources



"""
split_on_property_type
======================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class split_on_property_type(Operator):
    """Splits a given scoping or the mesh scoping (nodal or elemental) on
    given properties (elshape and/or material) and returns a scopings
    container with those split scopings.

    Parameters
    ----------
    mesh_scoping : Scoping, optional
        Scoping
    mesh : MeshedRegion
        Mesh region
    requested_location : str
        Location (default is elemental)
    skin_case : int, optional
        Set to 0: to have skin elements in their own
        group, 1: merge skin and solid
        elements, 2: merge skin and shell
        elements (default)
    label1 : str, optional
        Properties to apply the filtering 'mat'
        and/or 'elshape' (default is
        'elshape')
    label2 : str, optional
        Properties to apply the filtering 'mat'
        and/or 'elshape' (default is
        'elshape')


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.split_on_property_type()

    >>> # Make input connections
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_skin_case = int()
    >>> op.inputs.skin_case.connect(my_skin_case)
    >>> my_label1 = str()
    >>> op.inputs.label1.connect(my_label1)
    >>> my_label2 = str()
    >>> op.inputs.label2.connect(my_label2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.split_on_property_type(
    ...     mesh_scoping=my_mesh_scoping,
    ...     mesh=my_mesh,
    ...     requested_location=my_requested_location,
    ...     skin_case=my_skin_case,
    ...     label1=my_label1,
    ...     label2=my_label2,
    ... )

    >>> # Get output data
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """

    def __init__(
        self,
        mesh_scoping=None,
        mesh=None,
        requested_location=None,
        skin_case=None,
        label1=None,
        label2=None,
        config=None,
        server=None,
    ):
        super().__init__(name="scoping::by_property", config=config, server=server)
        self._inputs = InputsSplitOnPropertyType(self)
        self._outputs = OutputsSplitOnPropertyType(self)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if requested_location is not None:
            self.inputs.requested_location.connect(requested_location)
        if skin_case is not None:
            self.inputs.skin_case.connect(skin_case)
        if label1 is not None:
            self.inputs.label1.connect(label1)
        if label2 is not None:
            self.inputs.label2.connect(label2)

    @staticmethod
    def _spec():
        description = """Splits a given scoping or the mesh scoping (nodal or elemental) on
            given properties (elshape and/or material) and returns a
            scopings container with those split scopings."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""Scoping""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""Mesh region""",
                ),
                9: PinSpecification(
                    name="requested_location",
                    type_names=["string"],
                    optional=False,
                    document="""Location (default is elemental)""",
                ),
                12: PinSpecification(
                    name="skin_case",
                    type_names=["int32"],
                    optional=True,
                    document="""Set to 0: to have skin elements in their own
        group, 1: merge skin and solid
        elements, 2: merge skin and shell
        elements (default)""",
                ),
                13: PinSpecification(
                    name="label",
                    type_names=["string"],
                    optional=True,
                    document="""Properties to apply the filtering 'mat'
        and/or 'elshape' (default is
        'elshape')""",
                ),
                14: PinSpecification(
                    name="label",
                    type_names=["string"],
                    optional=True,
                    document="""Properties to apply the filtering 'mat'
        and/or 'elshape' (default is
        'elshape')""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scopings_container"],
                    optional=False,
                    document="""Scoping""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="scoping::by_property", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSplitOnPropertyType
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsSplitOnPropertyType
        """
        return super().outputs


class InputsSplitOnPropertyType(_Inputs):
    """Intermediate class used to connect user inputs to
    split_on_property_type operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.split_on_property_type()
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_skin_case = int()
    >>> op.inputs.skin_case.connect(my_skin_case)
    >>> my_label1 = str()
    >>> op.inputs.label1.connect(my_label1)
    >>> my_label2 = str()
    >>> op.inputs.label2.connect(my_label2)
    """

    def __init__(self, op: Operator):
        super().__init__(split_on_property_type._spec().inputs, op)
        self._mesh_scoping = Input(
            split_on_property_type._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mesh_scoping)
        self._mesh = Input(split_on_property_type._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)
        self._requested_location = Input(
            split_on_property_type._spec().input_pin(9), 9, op, -1
        )
        self._inputs.append(self._requested_location)
        self._skin_case = Input(
            split_on_property_type._spec().input_pin(12), 12, op, -1
        )
        self._inputs.append(self._skin_case)
        self._label1 = Input(split_on_property_type._spec().input_pin(13), 13, op, 0)
        self._inputs.append(self._label1)
        self._label2 = Input(split_on_property_type._spec().input_pin(14), 14, op, 1)
        self._inputs.append(self._label2)

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Scoping

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.split_on_property_type()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Mesh region

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.split_on_property_type()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator.

        Location (default is elemental)

        Parameters
        ----------
        my_requested_location : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.split_on_property_type()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> # or
        >>> op.inputs.requested_location(my_requested_location)
        """
        return self._requested_location

    @property
    def skin_case(self):
        """Allows to connect skin_case input to the operator.

        Set to 0: to have skin elements in their own
        group, 1: merge skin and solid
        elements, 2: merge skin and shell
        elements (default)

        Parameters
        ----------
        my_skin_case : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.split_on_property_type()
        >>> op.inputs.skin_case.connect(my_skin_case)
        >>> # or
        >>> op.inputs.skin_case(my_skin_case)
        """
        return self._skin_case

    @property
    def label1(self):
        """Allows to connect label1 input to the operator.

        Properties to apply the filtering 'mat'
        and/or 'elshape' (default is
        'elshape')

        Parameters
        ----------
        my_label1 : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.split_on_property_type()
        >>> op.inputs.label1.connect(my_label1)
        >>> # or
        >>> op.inputs.label1(my_label1)
        """
        return self._label1

    @property
    def label2(self):
        """Allows to connect label2 input to the operator.

        Properties to apply the filtering 'mat'
        and/or 'elshape' (default is
        'elshape')

        Parameters
        ----------
        my_label2 : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.split_on_property_type()
        >>> op.inputs.label2.connect(my_label2)
        >>> # or
        >>> op.inputs.label2(my_label2)
        """
        return self._label2


class OutputsSplitOnPropertyType(_Outputs):
    """Intermediate class used to get outputs from
    split_on_property_type operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.split_on_property_type()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """

    def __init__(self, op: Operator):
        super().__init__(split_on_property_type._spec().outputs, op)
        self._mesh_scoping = Output(split_on_property_type._spec().output_pin(0), 0, op)
        self._outputs.append(self._mesh_scoping)

    @property
    def mesh_scoping(self):
        """Allows to get mesh_scoping output of the operator

        Returns
        ----------
        my_mesh_scoping : ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.split_on_property_type()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_scoping = op.outputs.mesh_scoping()
        """  # noqa: E501
        return self._mesh_scoping



from ansys.dpf.gate import capi, external_operator_capi
from enum import Enum

from ansys.dpf.core import (
    field,
    property_field,
    scoping,
    collection,
    data_sources,
    meshed_region,
    time_freq_support,
    workflow,
    data_tree,
    dpf_operator,
    fields_container,
    scopings_container,
    meshes_container,
    result_info,
    string_field,
    custom_type_field,
)

external_operator_api = external_operator_capi.ExternalOperatorCAPI

functions_registry = []


@capi.OperatorCallBack
def __operator_main__(operator_functor, data):
    capi.OperatorMainCallback(operator_functor)(data)


_type_to_output_method = [
    (bool, external_operator_api.external_operator_put_out_bool),  # always before int
    ((int, Enum), external_operator_api.external_operator_put_out_int),
    (str, external_operator_api.external_operator_put_out_string),
    (float, external_operator_api.external_operator_put_out_double),
    (field.Field, external_operator_api.external_operator_put_out_field),
    (
        property_field.PropertyField,
        external_operator_api.external_operator_put_out_property_field,
    ),
    (
        string_field.StringField,
        external_operator_api.external_operator_put_out_string_field,
    ),
    (scoping.Scoping, external_operator_api.external_operator_put_out_scoping),
    (collection.Collection, external_operator_api.external_operator_put_out_collection),
    (
        data_sources.DataSources,
        external_operator_api.external_operator_put_out_data_sources,
    ),
    (
        meshed_region.MeshedRegion,
        external_operator_api.external_operator_put_out_meshed_region,
    ),
    (
        result_info.ResultInfo,
        external_operator_api.external_operator_put_out_result_info,
    ),
    (
        time_freq_support.TimeFreqSupport,
        external_operator_api.external_operator_put_out_time_freq,
    ),
    # TO DO : (cyclic_support.CyclicSupport, external_operator_api.external_operator_put_out_cy),
    (workflow.Workflow, external_operator_api.external_operator_put_out_workflow),
    (data_tree.DataTree, external_operator_api.external_operator_put_out_data_tree),
    (dpf_operator.Operator, external_operator_api.external_operator_put_out_operator),
    (
        custom_type_field.CustomTypeField,
        external_operator_api.external_operator_put_out_custom_type_field,
    ),
]

_type_to_input_method = [
    (bool, external_operator_api.external_operator_get_in_bool),
    (int, external_operator_api.external_operator_get_in_int),
    (str, external_operator_api.external_operator_get_in_string),
    (float, external_operator_api.external_operator_get_in_double),
    (field.Field, external_operator_api.external_operator_get_in_field, "field"),
    (
        property_field.PropertyField,
        external_operator_api.external_operator_get_in_property_field,
        "property_field",
    ),
    (
        string_field.StringField,
        external_operator_api.external_operator_get_in_string_field,
        "string_field",
    ),
    (
        custom_type_field.CustomTypeField,
        external_operator_api.external_operator_get_in_custom_type_field,
        "field",
    ),
    (
        scoping.Scoping,
        external_operator_api.external_operator_get_in_scoping,
        "scoping",
    ),
    (
        fields_container.FieldsContainer,
        external_operator_api.external_operator_get_in_fields_container,
        "fields_container",
    ),
    (
        scopings_container.ScopingsContainer,
        external_operator_api.external_operator_get_in_scopings_container,
        "scopings_container",
    ),
    (
        meshes_container.MeshesContainer,
        external_operator_api.external_operator_get_in_meshes_container,
        "meshes_container",
    ),
    (
        data_sources.DataSources,
        external_operator_api.external_operator_get_in_data_sources,
        "data_sources",
    ),
    (
        meshed_region.MeshedRegion,
        external_operator_api.external_operator_get_in_meshed_region,
        "mesh",
    ),
    # TO DO : (result_info.ResultInfo, external_operator_api.external_operator_get_in_re),
    (
        time_freq_support.TimeFreqSupport,
        external_operator_api.external_operator_get_in_time_freq,
        "time_freq_support",
    ),
    # TO DO : (cyclic_support.CyclicSupport, external_operator_api.external_operator_get_in_cy),
    (
        workflow.Workflow,
        external_operator_api.external_operator_get_in_workflow,
        "workflow",
    ),
    (
        data_tree.DataTree,
        external_operator_api.external_operator_get_in_data_tree,
        "data_tree",
    ),
    # TO DO : (dpf_operator.Operator, external_operator_api.external_operator_get_in_operator,
    # "operator"),
]



"""
compute_invariant_terms_motion
==============================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class compute_invariant_terms_motion(Operator):
    """Set the required data for the invariant terms computation (reduced
    matrices, lumped mass matrix, modes ...)

    Parameters
    ----------
    rom_matrices : FieldsContainer
        Fieldscontainers containing the reduced
        matrices
    mode_shapes : FieldsContainer
        Fieldscontainers containing the mode shapes,
        which are cst and nor for the cms
        method
    lumped_mass : FieldsContainer
        Fieldscontainers containing the lumped mass
    model_data : FieldsContainer
        Data describing the finite element model
    field_coordinates : Field
        Coordinates of all nodes
    nod :


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.compute_invariant_terms_motion()

    >>> # Make input connections
    >>> my_rom_matrices = dpf.FieldsContainer()
    >>> op.inputs.rom_matrices.connect(my_rom_matrices)
    >>> my_mode_shapes = dpf.FieldsContainer()
    >>> op.inputs.mode_shapes.connect(my_mode_shapes)
    >>> my_lumped_mass = dpf.FieldsContainer()
    >>> op.inputs.lumped_mass.connect(my_lumped_mass)
    >>> my_model_data = dpf.FieldsContainer()
    >>> op.inputs.model_data.connect(my_model_data)
    >>> my_field_coordinates = dpf.Field()
    >>> op.inputs.field_coordinates.connect(my_field_coordinates)
    >>> my_nod = dpf.()
    >>> op.inputs.nod.connect(my_nod)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.compute_invariant_terms_motion(
    ...     rom_matrices=my_rom_matrices,
    ...     mode_shapes=my_mode_shapes,
    ...     lumped_mass=my_lumped_mass,
    ...     model_data=my_model_data,
    ...     field_coordinates=my_field_coordinates,
    ...     nod=my_nod,
    ... )

    >>> # Get output data
    >>> result_model_data = op.outputs.model_data()
    >>> result_mode_shapes = op.outputs.mode_shapes()
    >>> result_lumped_mass = op.outputs.lumped_mass()
    >>> result_field_coordinates_and_euler_angles = op.outputs.field_coordinates_and_euler_angles()
    >>> result_nod = op.outputs.nod()
    >>> result_used_node_index = op.outputs.used_node_index()
    >>> result_eigenvalue = op.outputs.eigenvalue()
    >>> result_translational_mode_shape = op.outputs.translational_mode_shape()
    >>> result_rotational_mode_shape = op.outputs.rotational_mode_shape()
    >>> result_invrt_1 = op.outputs.invrt_1()
    >>> result_invrt_2 = op.outputs.invrt_2()
    >>> result_invrt_3 = op.outputs.invrt_3()
    >>> result_invrt_4 = op.outputs.invrt_4()
    >>> result_invrt_5 = op.outputs.invrt_5()
    >>> result_invrt_6 = op.outputs.invrt_6()
    >>> result_invrt_7 = op.outputs.invrt_7()
    >>> result_invrt_8 = op.outputs.invrt_8()
    """

    def __init__(
        self,
        rom_matrices=None,
        mode_shapes=None,
        lumped_mass=None,
        model_data=None,
        field_coordinates=None,
        nod=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="compute_invariant_terms_motion", config=config, server=server
        )
        self._inputs = InputsComputeInvariantTermsMotion(self)
        self._outputs = OutputsComputeInvariantTermsMotion(self)
        if rom_matrices is not None:
            self.inputs.rom_matrices.connect(rom_matrices)
        if mode_shapes is not None:
            self.inputs.mode_shapes.connect(mode_shapes)
        if lumped_mass is not None:
            self.inputs.lumped_mass.connect(lumped_mass)
        if model_data is not None:
            self.inputs.model_data.connect(model_data)
        if field_coordinates is not None:
            self.inputs.field_coordinates.connect(field_coordinates)
        if nod is not None:
            self.inputs.nod.connect(nod)

    @staticmethod
    def _spec():
        description = """Set the required data for the invariant terms computation (reduced
            matrices, lumped mass matrix, modes ...)"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="rom_matrices",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the reduced
        matrices""",
                ),
                1: PinSpecification(
                    name="mode_shapes",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the mode shapes,
        which are cst and nor for the cms
        method""",
                ),
                2: PinSpecification(
                    name="lumped_mass",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the lumped mass""",
                ),
                3: PinSpecification(
                    name="model_data",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Data describing the finite element model""",
                ),
                4: PinSpecification(
                    name="field_coordinates",
                    type_names=["field"],
                    optional=False,
                    document="""Coordinates of all nodes""",
                ),
                5: PinSpecification(
                    name="nod",
                    type_names=["vector<int32>"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="model_data",
                    type_names=["property_field"],
                    optional=False,
                    document="""Data describing the finite element model""",
                ),
                1: PinSpecification(
                    name="mode_shapes",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the mode shapes,
        which are cst and nor for the cms
        method""",
                ),
                2: PinSpecification(
                    name="lumped_mass",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fieldscontainers containing the lumped mass""",
                ),
                3: PinSpecification(
                    name="field_coordinates_and_euler_angles",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Coordinates and euler angles of all nodes""",
                ),
                4: PinSpecification(
                    name="nod",
                    type_names=["vector<int32>"],
                    optional=False,
                    document="""""",
                ),
                5: PinSpecification(
                    name="used_node_index",
                    type_names=["vector<int32>"],
                    optional=False,
                    document="""""",
                ),
                6: PinSpecification(
                    name="eigenvalue",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                7: PinSpecification(
                    name="translational_mode_shape",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                8: PinSpecification(
                    name="rotational_mode_shape",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                9: PinSpecification(
                    name="invrt_1",
                    type_names=["double"],
                    optional=False,
                    document="""""",
                ),
                10: PinSpecification(
                    name="invrt_2",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                11: PinSpecification(
                    name="invrt_3",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                12: PinSpecification(
                    name="invrt_4",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                13: PinSpecification(
                    name="invrt_5",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                14: PinSpecification(
                    name="invrt_6",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                15: PinSpecification(
                    name="invrt_7",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
                16: PinSpecification(
                    name="invrt_8",
                    type_names=["vector<double>"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="compute_invariant_terms_motion", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsComputeInvariantTermsMotion
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsComputeInvariantTermsMotion
        """
        return super().outputs


class InputsComputeInvariantTermsMotion(_Inputs):
    """Intermediate class used to connect user inputs to
    compute_invariant_terms_motion operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.compute_invariant_terms_motion()
    >>> my_rom_matrices = dpf.FieldsContainer()
    >>> op.inputs.rom_matrices.connect(my_rom_matrices)
    >>> my_mode_shapes = dpf.FieldsContainer()
    >>> op.inputs.mode_shapes.connect(my_mode_shapes)
    >>> my_lumped_mass = dpf.FieldsContainer()
    >>> op.inputs.lumped_mass.connect(my_lumped_mass)
    >>> my_model_data = dpf.FieldsContainer()
    >>> op.inputs.model_data.connect(my_model_data)
    >>> my_field_coordinates = dpf.Field()
    >>> op.inputs.field_coordinates.connect(my_field_coordinates)
    >>> my_nod = dpf.()
    >>> op.inputs.nod.connect(my_nod)
    """

    def __init__(self, op: Operator):
        super().__init__(compute_invariant_terms_motion._spec().inputs, op)
        self._rom_matrices = Input(
            compute_invariant_terms_motion._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._rom_matrices)
        self._mode_shapes = Input(
            compute_invariant_terms_motion._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mode_shapes)
        self._lumped_mass = Input(
            compute_invariant_terms_motion._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._lumped_mass)
        self._model_data = Input(
            compute_invariant_terms_motion._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._model_data)
        self._field_coordinates = Input(
            compute_invariant_terms_motion._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._field_coordinates)
        self._nod = Input(
            compute_invariant_terms_motion._spec().input_pin(5), 5, op, -1
        )
        self._inputs.append(self._nod)

    @property
    def rom_matrices(self):
        """Allows to connect rom_matrices input to the operator.

        Fieldscontainers containing the reduced
        matrices

        Parameters
        ----------
        my_rom_matrices : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> op.inputs.rom_matrices.connect(my_rom_matrices)
        >>> # or
        >>> op.inputs.rom_matrices(my_rom_matrices)
        """
        return self._rom_matrices

    @property
    def mode_shapes(self):
        """Allows to connect mode_shapes input to the operator.

        Fieldscontainers containing the mode shapes,
        which are cst and nor for the cms
        method

        Parameters
        ----------
        my_mode_shapes : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> op.inputs.mode_shapes.connect(my_mode_shapes)
        >>> # or
        >>> op.inputs.mode_shapes(my_mode_shapes)
        """
        return self._mode_shapes

    @property
    def lumped_mass(self):
        """Allows to connect lumped_mass input to the operator.

        Fieldscontainers containing the lumped mass

        Parameters
        ----------
        my_lumped_mass : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> op.inputs.lumped_mass.connect(my_lumped_mass)
        >>> # or
        >>> op.inputs.lumped_mass(my_lumped_mass)
        """
        return self._lumped_mass

    @property
    def model_data(self):
        """Allows to connect model_data input to the operator.

        Data describing the finite element model

        Parameters
        ----------
        my_model_data : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> op.inputs.model_data.connect(my_model_data)
        >>> # or
        >>> op.inputs.model_data(my_model_data)
        """
        return self._model_data

    @property
    def field_coordinates(self):
        """Allows to connect field_coordinates input to the operator.

        Coordinates of all nodes

        Parameters
        ----------
        my_field_coordinates : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> op.inputs.field_coordinates.connect(my_field_coordinates)
        >>> # or
        >>> op.inputs.field_coordinates(my_field_coordinates)
        """
        return self._field_coordinates

    @property
    def nod(self):
        """Allows to connect nod input to the operator.

        Parameters
        ----------
        my_nod :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> op.inputs.nod.connect(my_nod)
        >>> # or
        >>> op.inputs.nod(my_nod)
        """
        return self._nod


class OutputsComputeInvariantTermsMotion(_Outputs):
    """Intermediate class used to get outputs from
    compute_invariant_terms_motion operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.compute_invariant_terms_motion()
    >>> # Connect inputs : op.inputs. ...
    >>> result_model_data = op.outputs.model_data()
    >>> result_mode_shapes = op.outputs.mode_shapes()
    >>> result_lumped_mass = op.outputs.lumped_mass()
    >>> result_field_coordinates_and_euler_angles = op.outputs.field_coordinates_and_euler_angles()
    >>> result_nod = op.outputs.nod()
    >>> result_used_node_index = op.outputs.used_node_index()
    >>> result_eigenvalue = op.outputs.eigenvalue()
    >>> result_translational_mode_shape = op.outputs.translational_mode_shape()
    >>> result_rotational_mode_shape = op.outputs.rotational_mode_shape()
    >>> result_invrt_1 = op.outputs.invrt_1()
    >>> result_invrt_2 = op.outputs.invrt_2()
    >>> result_invrt_3 = op.outputs.invrt_3()
    >>> result_invrt_4 = op.outputs.invrt_4()
    >>> result_invrt_5 = op.outputs.invrt_5()
    >>> result_invrt_6 = op.outputs.invrt_6()
    >>> result_invrt_7 = op.outputs.invrt_7()
    >>> result_invrt_8 = op.outputs.invrt_8()
    """

    def __init__(self, op: Operator):
        super().__init__(compute_invariant_terms_motion._spec().outputs, op)
        self._model_data = Output(
            compute_invariant_terms_motion._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._model_data)
        self._mode_shapes = Output(
            compute_invariant_terms_motion._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._mode_shapes)
        self._lumped_mass = Output(
            compute_invariant_terms_motion._spec().output_pin(2), 2, op
        )
        self._outputs.append(self._lumped_mass)
        self._field_coordinates_and_euler_angles = Output(
            compute_invariant_terms_motion._spec().output_pin(3), 3, op
        )
        self._outputs.append(self._field_coordinates_and_euler_angles)
        self._nod = Output(compute_invariant_terms_motion._spec().output_pin(4), 4, op)
        self._outputs.append(self._nod)
        self._used_node_index = Output(
            compute_invariant_terms_motion._spec().output_pin(5), 5, op
        )
        self._outputs.append(self._used_node_index)
        self._eigenvalue = Output(
            compute_invariant_terms_motion._spec().output_pin(6), 6, op
        )
        self._outputs.append(self._eigenvalue)
        self._translational_mode_shape = Output(
            compute_invariant_terms_motion._spec().output_pin(7), 7, op
        )
        self._outputs.append(self._translational_mode_shape)
        self._rotational_mode_shape = Output(
            compute_invariant_terms_motion._spec().output_pin(8), 8, op
        )
        self._outputs.append(self._rotational_mode_shape)
        self._invrt_1 = Output(
            compute_invariant_terms_motion._spec().output_pin(9), 9, op
        )
        self._outputs.append(self._invrt_1)
        self._invrt_2 = Output(
            compute_invariant_terms_motion._spec().output_pin(10), 10, op
        )
        self._outputs.append(self._invrt_2)
        self._invrt_3 = Output(
            compute_invariant_terms_motion._spec().output_pin(11), 11, op
        )
        self._outputs.append(self._invrt_3)
        self._invrt_4 = Output(
            compute_invariant_terms_motion._spec().output_pin(12), 12, op
        )
        self._outputs.append(self._invrt_4)
        self._invrt_5 = Output(
            compute_invariant_terms_motion._spec().output_pin(13), 13, op
        )
        self._outputs.append(self._invrt_5)
        self._invrt_6 = Output(
            compute_invariant_terms_motion._spec().output_pin(14), 14, op
        )
        self._outputs.append(self._invrt_6)
        self._invrt_7 = Output(
            compute_invariant_terms_motion._spec().output_pin(15), 15, op
        )
        self._outputs.append(self._invrt_7)
        self._invrt_8 = Output(
            compute_invariant_terms_motion._spec().output_pin(16), 16, op
        )
        self._outputs.append(self._invrt_8)

    @property
    def model_data(self):
        """Allows to get model_data output of the operator

        Returns
        ----------
        my_model_data : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_model_data = op.outputs.model_data()
        """  # noqa: E501
        return self._model_data

    @property
    def mode_shapes(self):
        """Allows to get mode_shapes output of the operator

        Returns
        ----------
        my_mode_shapes : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mode_shapes = op.outputs.mode_shapes()
        """  # noqa: E501
        return self._mode_shapes

    @property
    def lumped_mass(self):
        """Allows to get lumped_mass output of the operator

        Returns
        ----------
        my_lumped_mass : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_lumped_mass = op.outputs.lumped_mass()
        """  # noqa: E501
        return self._lumped_mass

    @property
    def field_coordinates_and_euler_angles(self):
        """Allows to get field_coordinates_and_euler_angles output of the operator

        Returns
        ----------
        my_field_coordinates_and_euler_angles : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_coordinates_and_euler_angles = op.outputs.field_coordinates_and_euler_angles()
        """  # noqa: E501
        return self._field_coordinates_and_euler_angles

    @property
    def nod(self):
        """Allows to get nod output of the operator

        Returns
        ----------
        my_nod :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_nod = op.outputs.nod()
        """  # noqa: E501
        return self._nod

    @property
    def used_node_index(self):
        """Allows to get used_node_index output of the operator

        Returns
        ----------
        my_used_node_index :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_used_node_index = op.outputs.used_node_index()
        """  # noqa: E501
        return self._used_node_index

    @property
    def eigenvalue(self):
        """Allows to get eigenvalue output of the operator

        Returns
        ----------
        my_eigenvalue :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_eigenvalue = op.outputs.eigenvalue()
        """  # noqa: E501
        return self._eigenvalue

    @property
    def translational_mode_shape(self):
        """Allows to get translational_mode_shape output of the operator

        Returns
        ----------
        my_translational_mode_shape :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_translational_mode_shape = op.outputs.translational_mode_shape()
        """  # noqa: E501
        return self._translational_mode_shape

    @property
    def rotational_mode_shape(self):
        """Allows to get rotational_mode_shape output of the operator

        Returns
        ----------
        my_rotational_mode_shape :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_rotational_mode_shape = op.outputs.rotational_mode_shape()
        """  # noqa: E501
        return self._rotational_mode_shape

    @property
    def invrt_1(self):
        """Allows to get invrt_1 output of the operator

        Returns
        ----------
        my_invrt_1 : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_1 = op.outputs.invrt_1()
        """  # noqa: E501
        return self._invrt_1

    @property
    def invrt_2(self):
        """Allows to get invrt_2 output of the operator

        Returns
        ----------
        my_invrt_2 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_2 = op.outputs.invrt_2()
        """  # noqa: E501
        return self._invrt_2

    @property
    def invrt_3(self):
        """Allows to get invrt_3 output of the operator

        Returns
        ----------
        my_invrt_3 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_3 = op.outputs.invrt_3()
        """  # noqa: E501
        return self._invrt_3

    @property
    def invrt_4(self):
        """Allows to get invrt_4 output of the operator

        Returns
        ----------
        my_invrt_4 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_4 = op.outputs.invrt_4()
        """  # noqa: E501
        return self._invrt_4

    @property
    def invrt_5(self):
        """Allows to get invrt_5 output of the operator

        Returns
        ----------
        my_invrt_5 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_5 = op.outputs.invrt_5()
        """  # noqa: E501
        return self._invrt_5

    @property
    def invrt_6(self):
        """Allows to get invrt_6 output of the operator

        Returns
        ----------
        my_invrt_6 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_6 = op.outputs.invrt_6()
        """  # noqa: E501
        return self._invrt_6

    @property
    def invrt_7(self):
        """Allows to get invrt_7 output of the operator

        Returns
        ----------
        my_invrt_7 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_7 = op.outputs.invrt_7()
        """  # noqa: E501
        return self._invrt_7

    @property
    def invrt_8(self):
        """Allows to get invrt_8 output of the operator

        Returns
        ----------
        my_invrt_8 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_invariant_terms_motion()
        >>> # Connect inputs : op.inputs. ...
        >>> result_invrt_8 = op.outputs.invrt_8()
        """  # noqa: E501
        return self._invrt_8



"""
mesh_property_provider
======================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.outputs import _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mesh_property_provider(Operator):
    """Reads a property related to the mesh, defined by its name, by calling
    the readers defined by the data sources. These properties can be
    used to fill in the mesh.

    Parameters
    ----------
    mesh_scoping : Scoping, optional
        Retrieves a property field on a subset of
        elements or nodes.
    streams_container : StreamsContainer, optional
        Streams (result file container) (optional)
    data_sources : DataSources
        If the stream is null, retrieves the file
        path from the data sources.
    property_name : str
        Supported property names are: "mat",
        "named_selection",
        "named_selection_names",
        "apdl_element_type", "section",
        "elprops", "keyopt_1" to "keyopt_18".
    property_identifier : int or str, optional
        Retrieves a property at a given index or by
        name. for example, a named
        selection's number or a named
        selection's name.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.metadata.mesh_property_provider()

    >>> # Make input connections
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_identifier = int()
    >>> op.inputs.property_identifier.connect(my_property_identifier)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.metadata.mesh_property_provider(
    ...     mesh_scoping=my_mesh_scoping,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ...     property_name=my_property_name,
    ...     property_identifier=my_property_identifier,
    ... )

    >>> # Get output data
    >>> result_property = op.outputs.property()
    """

    def __init__(
        self,
        mesh_scoping=None,
        streams_container=None,
        data_sources=None,
        property_name=None,
        property_identifier=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mesh_property_provider", config=config, server=server)
        self._inputs = InputsMeshPropertyProvider(self)
        self._outputs = OutputsMeshPropertyProvider(self)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if property_name is not None:
            self.inputs.property_name.connect(property_name)
        if property_identifier is not None:
            self.inputs.property_identifier.connect(property_identifier)

    @staticmethod
    def _spec():
        description = """Reads a property related to the mesh, defined by its name, by calling
            the readers defined by the data sources. These properties
            can be used to fill in the mesh."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""Retrieves a property field on a subset of
        elements or nodes.""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Streams (result file container) (optional)""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""If the stream is null, retrieves the file
        path from the data sources.""",
                ),
                13: PinSpecification(
                    name="property_name",
                    type_names=["string"],
                    optional=False,
                    document="""Supported property names are: "mat",
        "named_selection",
        "named_selection_names",
        "apdl_element_type", "section",
        "elprops", "keyopt_1" to "keyopt_18".""",
                ),
                17: PinSpecification(
                    name="property_identifier",
                    type_names=["int32", "string"],
                    optional=True,
                    document="""Retrieves a property at a given index or by
        name. for example, a named
        selection's number or a named
        selection's name.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="property",
                    type_names=["scoping", "property_field", "string_field"],
                    optional=False,
                    document="""Returns a property field for properties:
        "mat", "apdl_element_type",
        "section", "elprops", "keyopt_1" to
        "keyopt_18" (or any mesh's property
        field), a scoping for
        properties:"named_selection", a
        string field for properties:
        "named_selection_names".""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mesh_property_provider", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshPropertyProvider
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMeshPropertyProvider
        """
        return super().outputs


class InputsMeshPropertyProvider(_Inputs):
    """Intermediate class used to connect user inputs to
    mesh_property_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.metadata.mesh_property_provider()
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_identifier = int()
    >>> op.inputs.property_identifier.connect(my_property_identifier)
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_property_provider._spec().inputs, op)
        self._mesh_scoping = Input(
            mesh_property_provider._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._mesh_scoping)
        self._streams_container = Input(
            mesh_property_provider._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._streams_container)
        self._data_sources = Input(
            mesh_property_provider._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)
        self._property_name = Input(
            mesh_property_provider._spec().input_pin(13), 13, op, -1
        )
        self._inputs.append(self._property_name)
        self._property_identifier = Input(
            mesh_property_provider._spec().input_pin(17), 17, op, -1
        )
        self._inputs.append(self._property_identifier)

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Retrieves a property field on a subset of
        elements or nodes.

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.mesh_property_provider()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator.

        Streams (result file container) (optional)

        Parameters
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.mesh_property_provider()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        If the stream is null, retrieves the file
        path from the data sources.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.mesh_property_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def property_name(self):
        """Allows to connect property_name input to the operator.

        Supported property names are: "mat",
        "named_selection",
        "named_selection_names",
        "apdl_element_type", "section",
        "elprops", "keyopt_1" to "keyopt_18".

        Parameters
        ----------
        my_property_name : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.mesh_property_provider()
        >>> op.inputs.property_name.connect(my_property_name)
        >>> # or
        >>> op.inputs.property_name(my_property_name)
        """
        return self._property_name

    @property
    def property_identifier(self):
        """Allows to connect property_identifier input to the operator.

        Retrieves a property at a given index or by
        name. for example, a named
        selection's number or a named
        selection's name.

        Parameters
        ----------
        my_property_identifier : int or str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.mesh_property_provider()
        >>> op.inputs.property_identifier.connect(my_property_identifier)
        >>> # or
        >>> op.inputs.property_identifier(my_property_identifier)
        """
        return self._property_identifier


class OutputsMeshPropertyProvider(_Outputs):
    """Intermediate class used to get outputs from
    mesh_property_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.metadata.mesh_property_provider()
    >>> # Connect inputs : op.inputs. ...
    >>> result_property = op.outputs.property()
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_property_provider._spec().outputs, op)
        self.property_as_scoping = Output(
            _modify_output_spec_with_one_type(
                mesh_property_provider._spec().output_pin(0), "scoping"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_scoping)
        self.property_as_property_field = Output(
            _modify_output_spec_with_one_type(
                mesh_property_provider._spec().output_pin(0), "property_field"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_property_field)
        self.property_as_string_field = Output(
            _modify_output_spec_with_one_type(
                mesh_property_provider._spec().output_pin(0), "string_field"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_string_field)
